// Load data from JSON file
d3.json('data.json')
  .then((data) => {
    const svg = d3.select('svg');
    const width = +svg.attr('width');
    const height = +svg.attr('height');

    // Create simulation
    const simulation = d3
      .forceSimulation(data.nodes)
      .force(
        'link',
        d3
          .forceLink(data.links)
          .id((d) => d.id)
          .distance(300), // Adjusting distance for visualization
      )
      .force('charge', d3.forceManyBody().strength(-100))
      .force(
        'center',
        d3.forceCenter(width / 2, height / 2),
      );

    // Create links
    const links = svg
      .selectAll('.link')
      .data(data.links)
      .enter()
      .append('line')
      .attr('class', 'link')
      .style('stroke', '#999')
      .style('stroke-width', 2);
    //.style('stroke-width', (d) => Math.sqrt(d.capacity));

    // Create text elements for displaying capacity
    const linkLabels = svg
      .selectAll('.link-label')
      .data(data.links)
      .enter()
      .append('text')
      .attr('class', 'link-label')
      .text((d) => `${d.capacity} MW`)
      .style('font-size', '9px')
      .attr('text-anchor', 'middle');

    // Create nodes
    const nodes = svg
      .selectAll('.node')
      .data(data.nodes)
      .enter()
      .append('g')
      .attr('class', 'node');

    // Append circles to nodes
    nodes
      .append('circle')
      .attr('r', 9)
      .attr('fill', (d) => {
        switch (d.group) {
          case 1:
            return '#1f77b4'; // Power plants
          case 2:
            return 'red'; // Substations
          case 3:
            return '#2ca02c'; // Distribution centers
          default:
            return '#ccc'; // Default color
        }
      });

    // Append text labels to nodes
    nodes
      .append('text')
      .text((d) => d.id)
      .attr('dy', 4)
      .style('font-size', '10px')
      .attr('text-anchor', 'middle');

    // Function to handle mouseover event
    function handleMouseOver(d, i) {
      // Display node information
      console.log('Node ID:', d.id);
      console.log('Group:', d.group);

      // Display connection information
      const connectedLinks = data.links.filter(
        (link) =>
          link.source === d.id || link.target === d.id,
      );
      console.log('Connected Links:', connectedLinks);
    }

    // Function to handle mouseout event
    function handleMouseOut(d, i) {
      // Clear any displayed information if needed
    }

    // Tick function to update positions
    simulation.on('tick', () => {
      links
        .attr('x1', (d) => d.source.x)
        .attr('y1', (d) => d.source.y)
        .attr('x2', (d) => d.target.x)
        .attr('y2', (d) => d.target.y);

      linkLabels
        .attr('x', (d) => (d.source.x + d.target.x) / 2)
        .attr('y', (d) => (d.source.y + d.target.y) / 2)
        .attr('transform', function (d) {
          const angle =
            Math.atan2(
              d.target.y - d.source.y,
              d.target.x - d.source.x,
            ) *
            (180 / Math.PI);
          return `rotate(${angle}, ${d.source.x + (d.target.x - d.source.x) / 2}, ${d.source.y + (d.target.y - d.source.y) / 2})`;
        })
        .attr('dy', '0.35em')
        .style('font-size', '15px');

      nodes.attr(
        'transform',
        (d) => `translate(${d.x},${d.y})`,
      );
    });
  })
  .catch((error) => {
    console.error('Error loading data:', error);
  });
